import logging
from typing import Any, Dict, Optional

import requests

from agentipy.agent import SolanaAgentKit
from agentipy.utils.agentipy_proxy.utils import encrypt_private_key

logger = logging.getLogger(__name__)

class DriftManager:
    @staticmethod
    def create_drift_user_account(
        agent: SolanaAgentKit,
        deposit_amount: float,
        deposit_symbol: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Creates a Drift user account and deposits initial funds.
        """
        try:
            if not all([deposit_amount, deposit_symbol]):
                raise ValueError("Deposit amount and deposit symbol are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "depositAmount": deposit_amount,
                "depositSymbol": deposit_symbol,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/create-drift-user-account",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}
        
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during Drift user account creation: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error during Drift user account creation: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def deposit_to_drift_user_account(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
        is_repayment: Optional[bool] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Deposits funds into a Drift user account.
        """
        try:
            if not all([amount, symbol]):
                raise ValueError("Amount and symbol are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
                "isRepayment": is_repayment,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/deposit-to-drift-user-account",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}
        
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during Drift deposit: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error during Drift deposit: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def withdraw_from_drift_user_account(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
        is_borrow: Optional[bool] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Withdraws funds from a Drift user account.
        """
        try:
            if not all([amount, symbol]):
                raise ValueError("Amount and symbol are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
                "isBorrow": is_borrow,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/withdraw-from-drift-user-account",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}
        
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during Drift withdrawal: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error during Drift withdrawal: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def trade_using_drift_perp_account(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
        action: str,
        trade_type: str,
        price: Optional[float] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Executes a trade using a Drift perpetual account.
        """
        try:
            if not all([amount, symbol, action, trade_type]):
                raise ValueError("Amount, symbol, action, and trade type are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
                "action": action,
                "type": trade_type,
                "price": price,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/trade-using-drift-perp-account",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}
        
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during Drift perpetual trade: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error during Drift perpetual trade: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def check_if_drift_account_exists(
        agent: SolanaAgentKit,
    ) -> Optional[Dict[str, Any]]:
        """
        Checks if the user has an existing Drift account.
        """
        try:
            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/check-if-drift-account-exists",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while checking Drift account: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error while checking Drift account: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def drift_user_account_info(
        agent: SolanaAgentKit,
    ) -> Optional[Dict[str, Any]]:
        """
        Fetches the user's Drift account information.
        """
        try:
            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/drift-user-account-info",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while fetching Drift account info: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error while fetching Drift account info: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def get_available_drift_markets(
        agent: SolanaAgentKit,
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves available Drift markets.
        """
        try:
            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/get-available-drift-markets",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while fetching Drift markets: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error while fetching Drift markets: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def stake_to_drift_insurance_fund(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Stakes funds to the Drift insurance fund.
        """
        try:
            if not all([amount, symbol]):
                raise ValueError("Amount and symbol are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/stake-to-drift-insurance-fund",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while staking to Drift insurance fund: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while staking to Drift insurance fund: {error}", exc_info=True)
            return {"success": False, "error": str(error)}
    
    @staticmethod
    def request_unstake_from_drift_insurance_fund(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Requests an unstake from the Drift insurance fund.
        """
        try:
            if not all([amount, symbol]):
                raise ValueError("Amount and symbol are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/request-unstake-from-drift-insurance-fund",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while requesting unstake: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while requesting unstake: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def unstake_from_drift_insurance_fund(
        agent: SolanaAgentKit,
        symbol: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Unstakes from the Drift insurance fund.
        """
        try:
            if not symbol:
                raise ValueError("Symbol is required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "symbol": symbol,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/unstake-from-drift-insurance-fund",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while unstaking: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while unstaking: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def drift_swap_spot_token(
        agent: SolanaAgentKit,
        from_symbol: str,
        to_symbol: str,
        slippage: Optional[float] = None,
        to_amount: Optional[float] = None,
        from_amount: Optional[float] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Swaps a spot token on Drift.
        """
        try:
            if not all([from_symbol, to_symbol]):
                raise ValueError("From symbol and to symbol are required.")

            if (to_amount is None and from_amount is None) or (to_amount is not None and from_amount is not None):
                raise ValueError("Provide either 'from_amount' or 'to_amount', but not both.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            swap_params = {
                "fromSymbol": from_symbol,
                "toSymbol": to_symbol,
                "slippage": slippage,
                **({"toAmount": to_amount} if to_amount is not None else {"fromAmount": from_amount}),
            }

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                **swap_params,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/drift-swap-spot-token",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while swapping spot token: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while swapping spot token: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def get_drift_perp_market_funding_rate(
        agent: SolanaAgentKit,
        symbol: str,
        period: str = "year",
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the funding rate for a Drift perpetual market.
        """
        try:
            if not symbol.endswith("-PERP"):
                raise ValueError("Symbol must be in the format '<name>-PERP'.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "symbol": symbol,
                "period": period,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/get-drift-perp-market-funding-rate",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while retrieving funding rate: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while retrieving funding rate: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def get_drift_entry_quote_of_perp_trade(
        agent: SolanaAgentKit,
        amount: float,
        symbol: str,
        action: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the entry quote for a Drift perpetual trade.
        """
        try:
            if not all([amount, symbol, action]):
                raise ValueError("Amount, symbol, and action are required.")

            if not symbol.endswith("-PERP"):
                raise ValueError("Symbol must be in the format '<name>-PERP'.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "symbol": symbol,
                "action": action,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/get-drift-entry-quote-of-perp-trade",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while retrieving entry quote: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while retrieving entry quote: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def get_drift_lend_borrow_apy(
        agent: SolanaAgentKit,
        symbol: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves the lending and borrowing APY for a Drift market.
        """
        try:
            if not symbol:
                raise ValueError("Symbol is required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "symbol": symbol,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/get-drift-lend-borrow-apy",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while retrieving APY: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while retrieving APY: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def create_drift_vault(
        agent: SolanaAgentKit,
        name: str,
        market_name: str,
        redeem_period: int,
        max_tokens: int,
        min_deposit_amount: float,
        management_fee: float,
        profit_share: float,
        hurdle_rate: Optional[float] = None,
        permissioned: Optional[bool] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Creates a Drift vault.
        """
        try:
            if not all([name, market_name, redeem_period, max_tokens, min_deposit_amount, management_fee, profit_share]):
                raise ValueError("All vault parameters are required.")

            if not "-" in market_name:
                raise ValueError("Market name must be in the format '<name>-<name>'.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            vault_params = {
                "name": name,
                "marketName": market_name,
                "redeemPeriod": redeem_period,
                "maxTokens": max_tokens,
                "minDepositAmount": min_deposit_amount,
                "managementFee": management_fee,
                "profitShare": profit_share,
                "hurdleRate": hurdle_rate,
                "permissioned": permissioned,
            }

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                **vault_params,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/create-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while creating Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while creating Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def update_drift_vault_delegate(
        agent: SolanaAgentKit,
        vault: str,
        delegate_address: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Updates the delegate for a Drift vault.
        """
        try:
            if not all([vault, delegate_address]):
                raise ValueError("Vault and delegate address are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "vault": vault,
                "delegateAddress": delegate_address,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/update-drift-vault-delegate",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while updating vault delegate: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while updating vault delegate: {error}", exc_info=True)
            return {"success": False, "error": str(error)}
        
    @staticmethod
    def update_drift_vault(
        agent: SolanaAgentKit,
        vault_address: str,
        name: str,
        market_name: str,
        redeem_period: int,
        max_tokens: int,
        min_deposit_amount: float,
        management_fee: float,
        profit_share: float,
        hurdle_rate: Optional[float] = None,
        permissioned: Optional[bool] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Updates an existing Drift vault.
        """
        try:
            if not all([vault_address, name, market_name, redeem_period, max_tokens, min_deposit_amount, management_fee, profit_share]):
                raise ValueError("All vault parameters are required.")

            if "-" not in market_name:
                raise ValueError("Market name must be in the format '<name>-<name>'.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            vault_params = {
                "vaultAddress": vault_address,
                "name": name,
                "marketName": market_name,
                "redeemPeriod": redeem_period,
                "maxTokens": max_tokens,
                "minDepositAmount": min_deposit_amount,
                "managementFee": management_fee,
                "profitShare": profit_share,
                "hurdleRate": hurdle_rate,
                "permissioned": permissioned,
            }

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                **vault_params,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/update-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while updating Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while updating Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def get_drift_vault_info(
        agent: SolanaAgentKit,
        vault_name: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Retrieves information about a Drift vault.
        """
        try:
            if not vault_name:
                raise ValueError("Vault name is required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "vaultName": vault_name,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/get-drift-vault-info",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while retrieving Drift vault info: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while retrieving Drift vault info: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def deposit_into_drift_vault(
        agent: SolanaAgentKit,
        amount: float,
        vault: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Deposits funds into a Drift vault.
        """
        try:
            if not all([amount, vault]):
                raise ValueError("Amount and vault address are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "vault": vault,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/deposit-into-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while depositing into Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while depositing into Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def request_withdrawal_from_drift_vault(
        agent: SolanaAgentKit,
        amount: float,
        vault: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Requests a withdrawal from a Drift vault.
        """
        try:
            if not all([amount, vault]):
                raise ValueError("Amount and vault address are required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "vault": vault,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/request-withdrawal-from-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while requesting withdrawal from Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while requesting withdrawal from Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}
        
    @staticmethod
    def withdraw_from_drift_vault(
        agent: SolanaAgentKit,
        vault: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Withdraws funds from a Drift vault.
        """
        try:
            if not vault:
                raise ValueError("Vault address is required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "vault": vault,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/withdraw-from-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while withdrawing from Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while withdrawing from Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def derive_drift_vault_address(
        agent: SolanaAgentKit,
        name: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Derives a Drift vault address based on the vault name.
        """
        try:
            if not name:
                raise ValueError("Vault name is required.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "name": name,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/derive-drift-vault-address",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "value": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while deriving Drift vault address: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while deriving Drift vault address: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def trade_using_delegated_drift_vault(
        agent: SolanaAgentKit,
        vault: str,
        amount: float,
        symbol: str,
        action: str,
        trade_type: str,
        price: Optional[float] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Executes a trade using a delegated Drift vault.
        """
        try:
            if not all([vault, amount, symbol, action, trade_type]):
                raise ValueError("Vault, amount, symbol, action, and trade_type are required.")

            if action not in ["long", "short"]:
                raise ValueError("Invalid action. Must be 'long' or 'short'.")

            if trade_type not in ["market", "limit"]:
                raise ValueError("Invalid trade type. Must be 'market' or 'limit'.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            trade_params = {
                "vault": vault,
                "amount": amount,
                "symbol": symbol,
                "action": action,
                "type": trade_type,
                "price": price,
            }

            payload: Dict[str, Any] = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                **trade_params,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/drift/trade-using-delegated-drift-vault",
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            if data.get("success"):
                return {
                    "success": True,
                    "transaction": data.get("value"),
                    "message": data.get("message"),
                }
            else:
                return {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error while executing trade using delegated Drift vault: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except ValueError as value_error:
            logger.error(f"Validation error: {value_error}", exc_info=True)
            return {"success": False, "error": str(value_error)}
        except Exception as error:
            logger.error(f"Unexpected error while executing trade using delegated Drift vault: {error}", exc_info=True)
            return {"success": False, "error": str(error)}


import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaSNSResolveTool(BaseTool):
    name: str = "solana_sns_resolve"
    description: str = """
    Resolves a Solana Name Service (SNS) domain to its corresponding address.

    Input: A JSON string with:
    {
        "domain": "string, the SNS domain (e.g., example.sol)"
    }

    Output:
    {
        "address": "string, the resolved Solana address",
        "message": "string, if resolution fails"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "domain": {"type": str, "required": True}
            }
            validate_input(data, schema)

            domain = data["domain"]
            if not domain:
                raise ValueError("Domain is required.")

            address = await self.solana_kit.resolve_name_to_address(domain)
            return {
                "address": address or "Not Found",
                "message": "Success" if address else "Domain not found."
            }
        except Exception as e:
            return {
                "address": None,
                "message": f"Error resolving domain: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )




from mcp.types import Tool

from agentipy.tools.stake_with_jup import StakeManager
from agentipy.tools.trade import TradeManager

JUPITER_ACTIONS = {
    "STAKE_WITH_JUP": Tool(
        name="STAKE_WITH_JUP",
        description="Stake SOL with Jupiter. input_schema Example: { amount: number }",
        inputSchema={
            "amount": {
                "type": "number",
                "description": "Amount of SOL to stake"
            }
        },
        handler=lambda agent, params: StakeManager.stake_with_jup(
            agent,
            amount=params["amount"],
        ),
    ),
    "TRADE_WITH_JUP": Tool(
        name="TRADE_WITH_JUP",
        description="Trade a token with Jupiter. input_schema Example: { output_mint: string, input_amount: number, input_mint:string, slippage_bps: number = 50 }",
        inputSchema={
            "output_mint": {"type": "string", "description": "Token to trade"},
            "input_amount": {"type": "number", "description": "Amount of token to trade"},
            "input_mint": {"type": "string", "description": "Token to trade"},
            "slippage_bps": {"type": "number", "description": "Slippage in basis points"},
        },
        handler=lambda agent, params: TradeManager.trade(
            agent,
            output_mint=params["output_mint"],
            input_amount=params["input_amount"],
            input_mint=params["input_mint"],
            slippage_bps=params["slippage_bps"],
        ),
    ),
}



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class ElfaAiGetSmartTwitterAccountStatsTool(BaseTool):
    name: str = "elfa_ai_get_smart_twitter_account_stats"
    description: str = """
    Retrieves smart Twitter account statistics using ElfaAiManager.

    Input: A JSON string with:
    {
        "username": "string, the Twitter username"
    }
    Output:
    {
        "account_stats": "dict, the Twitter account statistics",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "username": {"type": str, "required": True}
            }
            validate_input(data, schema)
            account_stats = await self.agent_kit.get_smart_twitter_account_stats(
                username=data["username"]
            )
            return {
                "account_stats": account_stats,
                "message": "Success"
            }
        except Exception as e:
            return {
                "account_stats": None,
                "message": f"Error fetching smart Twitter account stats: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input



class SolanaGetMetaplexAssetsByCreatorTool(BaseTool):
    name: str = "solana_get_metaplex_assets_by_creator"
    description: str = """
    Fetches assets created by a specific creator.

    Input: A JSON string with:
    {
        "creator": "string, the creator's public key",
        "only_verified": "bool, fetch only verified assets (default: False)",
        "sort_by": "string, field to sort by (e.g., 'date')",
        "sort_direction": "string, 'asc' or 'desc'",
        "limit": "int, maximum number of assets",
        "page": "int, page number for paginated results"
    }

    Output:
    {
        "success": "bool, whether the operation was successful",
        "value": "list, the list of assets if successful",
        "message": "string, additional details or error information"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "creator": {"type": str, "required": True},
                "only_verified": {"type": bool, "required": False},
                "sort_by": {"type": str, "required": False},
                "sort_direction": {"type": str, "required": False},
                "limit": {"type": int, "required": False, "min": 1},
                "page": {"type": int, "required": False, "min": 1}
            }
            validate_input(data, schema)

            creator = data["creator"]
            only_verified = data.get("only_verified", False)
            sort_by = data.get("sort_by")
            sort_direction = data.get("sort_direction")
            limit = data.get("limit")
            page = data.get("page")

            result = await self.solana_kit.get_metaplex_assets_by_creator(
                creator=creator,
                onlyVerified=only_verified,
                sortBy=sort_by,
                sortDirection=sort_direction,
                limit=limit,
                page=page,
            )
            return result
        except Exception as e:
            return {"success": False, "message": f"Error fetching assets by creator: {str(e)}"}

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class SolanaGetMetaplexAssetsByAuthorityTool(BaseTool):
    name: str = "solana_get_metaplex_assets_by_authority"
    description: str = """
    Fetches assets created by a specific authority.

    Input: A JSON string with:
    {
        "authority": "string, the authority's public key",
        "sort_by": "string, field to sort by (e.g., 'date')",
        "sort_direction": "string, 'asc' or 'desc'",
        "limit": "int, maximum number of assets",
        "page": "int, page number for paginated results"
    }

    Output:
    {
        "success": "bool, whether the operation was successful",
        "value": "list, the list of assets if successful",
        "message": "string, additional details or error information"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "authority": {"type": str, "required": True},
                "sort_by": {"type": str, "required": False},
                "sort_direction": {"type": str, "required": False},
                "limit": {"type": int, "required": False, "min": 1},
                "page": {"type": int, "required": False, "min": 1}
            }
            validate_input(data, schema)

            authority = data["authority"]
            sort_by = data.get("sort_by")
            sort_direction = data.get("sort_direction")
            limit = data.get("limit")
            page = data.get("page")

            result = await self.solana_kit.get_metaplex_assets_by_authority(
                authority=authority,
                sortBy=sort_by,
                sortDirection=sort_direction,
                limit=limit,
                page=page,
            )
            return result
        except Exception as e:
            return {"success": False, "message": f"Error fetching assets by authority: {str(e)}"}

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class SolanaDeployCollectionTool(BaseTool):
    name: str = "solana_deploy_collection"
    description: str = """
    Deploys an NFT collection using the Metaplex program.

    Input: A JSON string with:
    {
        "name": "string, the name of the NFT collection",
        "uri": "string, the metadata URI",
        "royalty_basis_points": "int, royalty percentage in basis points (e.g., 500 for 5%)",
        "creator_address": "string, the creator's public key"
    }

    Output:
    {
        "success": "bool, whether the operation was successful",
        "value": "string, the transaction signature if successful",
        "message": "string, additional details or error information"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "name": {"type": str, "required": True},
                "uri": {"type": str, "required": True},
                "royalty_basis_points": {"type": int, "required": True, "min": 0, "max": 10000},
                "creator_address": {"type": str, "required": True}
            }
            validate_input(data, schema)

            name = data["name"]
            uri = data["uri"]
            royalty_basis_points = data["royalty_basis_points"]
            creator_address = data["creator_address"]

            result = await self.solana_kit.deploy_collection(
                name=name,
                uri=uri,
                royalty_basis_points=royalty_basis_points,
                creator_address=creator_address,
            )
            return result
        except Exception as e:
            return {"success": False, "message": f"Error deploying collection: {str(e)}"}

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class AlloraGetInferenceByTopicIdTool(BaseTool):
    name: str = "allora_get_inference_by_topic_id"
    description: str = """
    Fetches a price inference for BTC or ETH for a given timeframe (5m or 8h) from the Allora Network using AlloraManager.

    Input: A JSON string with:
    {
        "topic_id": "int, the topic ID to fetch inference for"
    }
    Output:
    {
        "inference_data": "dict, the inferred price and confidence interval",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "topic_id": {"type": int, "required": True}
            }
            validate_input(data, schema)
            inference_data = await self.agent_kit.get_inference_by_topic_id(
                topic_id=data["topic_id"]
            )
            return {
                "inference_data": inference_data,
                "message": "Success"
            }
        except Exception as e:
            return {
                "inference_data": None,
                "message": f"Error fetching price inference: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")


import json
from agentipy.agent import SolanaAgentKit
from langchain.tools import BaseTool
from agentipy.helpers import validate_input

class CreateDriftUserAccountTool(BaseTool):
    name: str = "create_drift_user_account"
    description: str = """
    Creates a Drift user account with an initial deposit.

    Input: A JSON string with:
    {
        "deposit_amount": "float, amount to deposit",
        "deposit_symbol": "string, symbol of the asset to deposit"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "deposit_amount": {"type": float, "required": True},
                "deposit_symbol": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
            transaction = await self.solana_kit.create_drift_user_account(
                deposit_amount=data["deposit_amount"],
                deposit_symbol=data["deposit_symbol"],
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error creating Drift user account: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class DepositToDriftUserAccountTool(BaseTool):
    name: str = "deposit_to_drift_user_account"
    description: str = """
    Deposits funds into a Drift user account.

    Input: A JSON string with:
    {
        "amount": "float, amount to deposit",
        "symbol": "string, symbol of the asset",
        "is_repayment": "bool, optional, whether the deposit is a loan repayment"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "amount": {"type": float, "required": True},
                "symbol": {"type": str, "required": True},
                "is_repayment": {"type": bool, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)
            
            transaction = await self.solana_kit.deposit_to_drift_user_account(
                amount=data["amount"],
                symbol=data["symbol"],
                is_repayment=data.get("is_repayment"),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error depositing to Drift user account: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")
    
class WithdrawFromDriftUserAccountTool(BaseTool):
    name: str = "withdraw_from_drift_user_account"
    description: str = """
    Withdraws funds from a Drift user account.

    Input: A JSON string with:
    {
        "amount": "float, amount to withdraw",
        "symbol": "string, symbol of the asset",
        "is_borrow": "bool, optional, whether the withdrawal is a borrow request"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "amount": {"type": float, "required": True},
                "symbol": {"type": str, "required": True},
                "is_borrow": {"type": bool, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)
          
            transaction = await self.solana_kit.withdraw_from_drift_user_account(
                amount=data["amount"],
                symbol=data["symbol"],
                is_borrow=data.get("is_borrow"),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error withdrawing from Drift user account: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class CheckIfDriftAccountExistsTool(BaseTool):
    name: str = "check_if_drift_account_exists"
    description: str = """
    Checks if a Drift user account exists.

    Input: None.
    Output:
    {
        "exists": "bool, whether the Drift user account exists",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            exists = await self.solana_kit.check_if_drift_account_exists()
            return {
                "exists": exists,
                "message": "Success"
            }
        except Exception as e:
            return {
                "exists": None,
                "message": f"Error checking Drift account existence: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class DriftUserAccountInfoTool(BaseTool):
    name: str = "drift_user_account_info"
    description: str = """
    Retrieves Drift user account information.

    Input: None.
    Output:
    {
        "account_info": "dict, account details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            account_info = await self.solana_kit.drift_user_account_info()
            return {
                "account_info": account_info,
                "message": "Success"
            }
        except Exception as e:
            return {
                "account_info": None,
                "message": f"Error fetching Drift user account info: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")




import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class Create3LandCollectionTool(BaseTool):
    name: str = "create_3land_collection"
    description: str = """
    Creates a 3Land NFT collection.

    Input: A JSON string with:
    {
        "collection_symbol": "string, symbol of the collection",
        "collection_name": "string, name of the collection",
        "collection_description": "string, description of the collection",
        "main_image_url": "string, optional, URL of the main image",
        "cover_image_url": "string, optional, URL of the cover image",
        "is_devnet": "bool, optional, whether to use devnet"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "collection_symbol": {"type": str, "required": True},
                "collection_name": {"type": str, "required": True},
                "collection_description": {"type": str, "required": True},
                "main_image_url": {"type": str, "required": False},
                "cover_image_url": {"type": str, "required": False},
                "is_devnet": {"type": bool, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)

            
            
            transaction = await self.solana_kit.create_3land_collection(
                collection_symbol=data["collection_symbol"],
                collection_name=data["collection_name"],
                collection_description=data["collection_description"],
                main_image_url=data.get("main_image_url"),
                cover_image_url=data.get("cover_image_url"),
                is_devnet=data.get("is_devnet", False),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error creating 3land collection: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



from agentipy.agent import SolanaAgentKit
from agentipy.langchain.cybersmanager.create_coin import SolanaCybersCreateCoinTool


def get_cyber_tools(solana_kit: SolanaAgentKit):
    return [
        SolanaCybersCreateCoinTool(solana_kit=solana_kit),
    ]  


from agentipy.langchain.solayermanager.solayer_restake import SolayerRestakeTool
from agentipy.agent import SolanaAgentKit


def get_solayer_tools(agent_kit: SolanaAgentKit):
    return [
        SolayerRestakeTool(agent_kit=agent_kit)
    ]   


import json
from agentipy.agent import SolanaAgentKit
from langchain.tools import BaseTool
from agentipy.helpers import validate_input

class SolanaGetBundleStatuses(BaseTool):
    name: str = "get_bundle_statuses"
    description: str = """
    Get the current statuses of specified Jito bundles.

    Input: A JSON string with:
    {
        "bundle_uuids": "List of bundle UUIDs"
    }

    Output:
    {
        "statuses": "List of corresponding bundle statuses"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "bundle_uuids": {"type": list, "required": True}
            }
            validate_input(data, schema)

            bundle_uuids = data["bundle_uuids"]
            result = await self.solana_kit.get_bundle_statuses(bundle_uuids)
            return {
                "statuses": result
            }
        except Exception as e:
            return {
                "statuses": None
            }

    def _run(self):
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )

class SolanaSendBundle(BaseTool):
    name: str = "send_bundle"
    description: str = """
    Send a bundle of transactions to the Jito network for processing.

    Input: A JSON string with:
    {
        "txn_signatures": "List of transaction signatures"
    }

    Output:
    {
        "bundle_ids": "List of unique identifiers for the submitted bundles"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "txn_signatures": {"type": list, "required": True}
            }
            validate_input(data, schema)

            txn_signatures = data["txn_signatures"]
            result = await self.solana_kit.send_bundle(txn_signatures)
            return {
                "bundle_ids": result
            }
        except Exception as e:
            return {
                "bundle_ids": None
            }

    def _run(self):
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )

class SolanaGetInflightBundleStatuses(BaseTool):
    name: str = "get_inflight_bundle_statuses"
    description: str = """
    Get the statuses of bundles that are currently in flight.

    Input: A JSON string with:
    {
        "bundle_uuids": "List of bundle UUIDs"
    }

    Output:
    {
        "statuses": "List of statuses corresponding to currently inflight bundles"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "bundle_uuids": {"type": list, "required": True}
            }
            validate_input(data, schema)

            bundle_uuids = data["bundle_uuids"]
            result = await self.solana_kit.get_inflight_bundle_statuses(bundle_uuids)
            return {
                "statuses": result
            }
        except Exception as e:
            return {
                "statuses": None
            }

    def _run(self):
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )





import json
from agentipy.agent import SolanaAgentKit
from langchain.tools import BaseTool
from agentipy.helpers import validate_input

class RockPaperScissorsTool(BaseTool):
    name: str = "rock_paper_scissors"
    description: str = """
    Plays a game of Rock-Paper-Scissors using RockPaperScissorsManager by Send Arcade.

    Input: A JSON string with:
    {
        "amount" "float, The amount of SOL to stake.
        "choice": "string, the player's choice ('rock', 'paper', or 'scissors')"
    }
    Output:
    {
        "result": "string, the game result",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "amount": {"type": float, "required": True},
                "choice": {"type": str, "required": True, }
            }
            data = json.loads(input)
            validate_input(data, schema)
            result = await self.agent_kit.rock_paper_scissors(
                amount=data["amount"],
                choice=data["choice"]
            )
            return {
                "result": result,
                "message": "Success"
            }
        except Exception as e:
            return {
                "result": None,
                "message": f"Error playing Rock-Paper-Scissors: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



from agentipy.agent import SolanaAgentKit
from agentipy.langchain.core.balance import SolanaBalanceTool
from agentipy.langchain.core.burn_and_close import SolanaBurnAndCloseTool
from agentipy.langchain.core.burn_and_close_multiple import \
    SolanaBurnAndCloseMultipleTool
from agentipy.langchain.core.create_image import SolanaCreateImageTool
from agentipy.langchain.core.data import (SolanaTokenDataByTickerTool,
                                          SolanaTokenDataTool)
from agentipy.langchain.core.deploy_token import SolanaDeployTokenTool
from agentipy.langchain.core.fetch_price import SolanaFetchPriceTool
from agentipy.langchain.core.get_wallet_address import \
    SolanaGetWalletAddressTool
from agentipy.langchain.core.report import (SolanaFetchTokenDetailedReportTool,
                                            SolanaFetchTokenReportSummaryTool)
from agentipy.langchain.core.stake import SolanaStakeTool
from agentipy.langchain.core.tps_calculator import SolanaTPSCalculatorTool
from agentipy.langchain.core.trade import SolanaTradeTool
from agentipy.langchain.core.transfer import SolanaTransferTool


def get_all_core_tools(solana_kit: SolanaAgentKit) -> list:
    """Get all core Solana tools initialized with the provided SolanaAgentKit.
    
    Args:
        solana_kit (SolanaAgentKit): The Solana agent kit to initialize tools with
        
    Returns:
        list: List of initialized Solana tools
    """
    return [
        SolanaBalanceTool(solana_kit=solana_kit),
        SolanaCreateImageTool(solana_kit=solana_kit),
        SolanaDeployTokenTool(solana_kit=solana_kit),
        SolanaTradeTool(solana_kit=solana_kit),
       
        SolanaStakeTool(solana_kit=solana_kit),
        SolanaGetWalletAddressTool(solana_kit=solana_kit),
        SolanaTPSCalculatorTool(solana_kit=solana_kit),
        SolanaFetchPriceTool(solana_kit=solana_kit),
        SolanaTokenDataTool(solana_kit=solana_kit),
        SolanaTokenDataByTickerTool(solana_kit=solana_kit),
        SolanaFetchTokenReportSummaryTool(solana_kit=solana_kit),
        SolanaFetchTokenDetailedReportTool(solana_kit=solana_kit),
        SolanaTransferTool(solana_kit=solana_kit),
        SolanaBurnAndCloseTool(solana_kit=solana_kit),
        SolanaBurnAndCloseMultipleTool(solana_kit=solana_kit),
    ]


import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input

class SolanaHeliusGetRawTransactionsTool(BaseTool):
    name: str = "solana_helius_get_raw_transactions"
    description: str = """
    Fetch raw transactions for a list of accounts.

    Input: A JSON string with:
    {
        "signatures": ["string, the transaction signatures"],
        "start_slot": "optional start slot",
        "end_slot": "optional end slot",
        "start_time": "optional start time",
        "end_time": "optional end time",
        "limit": "optional limit",
        "sort_order": "optional sort order",
        "pagination_token": "optional pagination token"
    }

    Output:
    {
        "transactions": "list of raw transactions"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "signatures": {"type": list, "required": True},
                "start_slot": {"type": int, "required": False},
                "end_slot": {"type": int, "required": False},
                "start_time": {"type": int, "required": False},
                "end_time": {"type": int, "required": False},
                "limit": {"type": int, "required": False},
                "sort_order": {"type": str, "required": False},
                "pagination_token": {"type": str, "required": False}
            }
            validate_input(data, schema)

            signatures = data["signatures"]
            start_slot = data.get("start_slot")
            end_slot = data.get("end_slot")
            start_time = data.get("start_time")
            end_time = data.get("end_time")
            limit = data.get("limit")
            sort_order = data.get("sort_order")
            pagination_token = data.get("pagination_token")

            result = await self.solana_kit.get_raw_transactions(
                signatures, start_slot, end_slot, start_time, end_time, limit, sort_order, pagination_token
            )
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )




import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class OpenPerpTradeShortTool(BaseTool):
    name: str = "open_perp_trade_short"
    description: str = """
    Opens a perpetual short trade.

    Input: A JSON string with:
    {
        "price": "float, entry price for the trade",
        "collateral_amount": "float, amount of collateral",
        "collateral_mint": "string, optional, mint address of the collateral",
        "leverage": "float, optional, leverage factor",
        "trade_mint": "string, optional, token mint address",
        "slippage": "float, optional, slippage tolerance"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "price": {"type": float, "required": True},
                "collateral_amount": {"type": float, "required": True},
                "collateral_mint": {"type": str, "required": False},
                "leverage": {"type": float, "required": False},
                "trade_mint": {"type": str, "required": False},
                "slippage": {"type": float, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)

            transaction = await self.solana_kit.open_perp_trade_short(  
                price=data["price"],
                collateral_amount=data["collateral_amount"],
                collateral_mint=data.get("collateral_mint"),
                leverage=data.get("leverage"),
                trade_mint=data.get("trade_mint"),
                slippage=data.get("slippage")
            )
           
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error opening perp short trade: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")
 



from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
import json
from agentipy.helpers import validate_input


class SolanaDeployTokenTool(BaseTool):
    name:str = "solana_deploy_token"
    description:str = """
    Deploy a new SPL token. Input should be JSON string with:
    {
        "decimals": 9,
        "initialSupply": 1000
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "decimals": {"type": int, "required": True, "min": 0, "max": 9},
                "initialSupply": {"type": int, "required": True, "min": 1}
            }
            validate_input(data, schema)
            decimals = data.get("decimals", 9)
            token_details = await self.solana_kit.deploy_token(decimals)
            return {
                "status": "success",
                "message": "Token deployed successfully",
                "mintAddress": token_details["mint"],
                "decimals": decimals,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
                "code": getattr(e, "code", "UNKNOWN_ERROR"),
            }
        
    def _run(self):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )





import json

from langchain.tools import BaseTool

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class ManifestCreateMarketTool(BaseTool):
    name: str = "manifest_create_market"
    description: str = """
    Creates a new market using ManifestManager.

    Input: A JSON string with:
    {
        "base_mint": "string, the base mint address",
        "quote_mint": "string, the quote mint address"
    }
    Output:
    {
        "market_data": "dict, the created market details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "base_mint": {"type": str, "required": True},
                "quote_mint": {"type": str, "required": True}
            }
            validate_input(data, schema)
            market_data = await self.solana_kit.create_manifest_market(
                base_mint=data["base_mint"],
                quote_mint=data["quote_mint"]
            )
            return {
                "market_data": market_data,
                "message": "Success"
            }
        except Exception as e:
            return {
                "market_data": None,
                "message": f"Error creating market: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class ManifestPlaceLimitOrderTool(BaseTool):
    name: str = "manifest_place_limit_order"
    description: str = """
    Places a limit order on a market using ManifestManager.

    Input: A JSON string with:
    {
        "market_id": "string, the market ID",
        "quantity": "float, the quantity to trade",
        "side": "string, 'buy' or 'sell'",
        "price": "float, the price per unit"
    }
    Output:
    {
        "order_details": "dict, the placed order details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "market_id": {"type": str, "required": True},
                "quantity": {"type": float, "required": True},
                "side": {"type": str, "required": True},
                "price": {"type": float, "required": True}
            }
            validate_input(data, schema)
            order_details = await self.solana_kit.place_limit_order(
                market_id=data["market_id"],
                quantity=data["quantity"],
                side=data["side"],
                price=data["price"]
            )
            return {
                "order_details": order_details,
                "message": "Success"
            }
        except Exception as e:
            return {
                "order_details": None,
                "message": f"Error placing limit order: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class ManifestPlaceBatchOrdersTool(BaseTool):
    name: str = "manifest_place_batch_orders"
    description: str = """
    Places multiple batch orders on a market using ManifestManager.

    Input: A JSON string with:
    {
        "market_id": "string, the market ID",
        "orders": "list, a list of orders (each order must include 'quantity', 'side', and 'price')"
    }
    Output:
    {
        "batch_order_details": "dict, details of the placed batch orders",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "market_id": {"type": str, "required": True},
                "orders": {"type": list, "required": True}
            }
            validate_input(data, schema)
            batch_order_details = await self.solana_kit.place_batch_orders(
                market_id=data["market_id"],
                orders=data["orders"]
            )
            return {
                "batch_order_details": batch_order_details,
                "message": "Success"
            }
        except Exception as e:
            return {
                "batch_order_details": None,
                "message": f"Error placing batch orders: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")




import json
from agentipy.agent import SolanaAgentKit
from langchain.tools import BaseTool
from agentipy.helpers import validate_input

class GetAvailableDriftMarketsTool(BaseTool):
    name: str = "get_available_drift_markets"
    description: str = """
    Retrieves available markets on Drift.

    Input: None.
    Output:
    {
        "markets": "dict, list of available Drift markets",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            markets = await self.solana_kit.get_available_drift_markets()
            return {
                "markets": markets,
                "message": "Success"
            }
        except Exception as e:
            return {
                "markets": None,
                "message": f"Error fetching available Drift markets: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class TradeUsingDriftPerpAccountTool(BaseTool):
    name: str = "trade_using_drift_perp_account"
    description: str = """
    Executes a trade using a Drift perpetual account.

    Input: A JSON string with:
    {
        "amount": "float, trade amount",
        "symbol": "string, market symbol",
        "action": "string, 'long' or 'short'",
        "trade_type": "string, 'market' or 'limit'",
        "price": "float, optional, trade execution price"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "amount": {"type": float, "required": True},
                "symbol": {"type": str, "required": True},
                "action": {"type": str, "required": True},
                "trade_type": {"type": str, "required": True},
                "price": {"type": float, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)
            transaction = await self.solana_kit.trade_using_drift_perp_account(
                amount=data["amount"],
                symbol=data["symbol"],
                action=data["action"],
                trade_type=data["trade_type"],
                price=data.get("price"),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error trading using Drift perp account: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class GetDriftPerpMarketFundingRateTool(BaseTool):
    name: str = "get_drift_perp_market_funding_rate"
    description: str = """
    Retrieves the funding rate for a Drift perpetual market.

    Input: A JSON string with:
    {
        "symbol": "string, market symbol (must end in '-PERP')",
        "period": "string, optional, funding rate period, either 'year' or 'hour' (default: 'year')"
    }
    Output:
    {
        "funding_rate": "dict, funding rate details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "symbol": {"type": str, "required": True},
                "period": {"type": str, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)            
            funding_rate = await self.solana_kit.get_drift_perp_market_funding_rate(
                symbol=data["symbol"],
                period=data.get("period", "year"),
            )
            return {
                "funding_rate": funding_rate,
                "message": "Success"
            }
        except Exception as e:
            return {
                "funding_rate": None,
                "message": f"Error getting Drift perp market funding rate: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class GetDriftEntryQuoteOfPerpTradeTool(BaseTool):
    name: str = "get_drift_entry_quote_of_perp_trade"
    description: str = """
    Retrieves the entry quote for a perpetual trade on Drift.

    Input: A JSON string with:
    {
        "amount": "float, trade amount",
        "symbol": "string, market symbol (must end in '-PERP')",
        "action": "string, 'long' or 'short'"
    }
    Output:
    {
        "entry_quote": "dict, entry quote details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "amount": {"type": float, "required": True},
                "symbol": {"type": str, "required": True},
                "action": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
            entry_quote = await self.solana_kit.get_drift_entry_quote_of_perp_trade(
                amount=data["amount"],
                symbol=data["symbol"],
                action=data["action"],
            )
            return {
                "entry_quote": entry_quote,
                "message": "Success"
            }
        except Exception as e:
            return {
                "entry_quote": None,
                "message": f"Error getting Drift entry quote of perp trade: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")

class DriftSwapSpotTokenTool(BaseTool):
    name: str = "drift_swap_spot_token"
    description: str = """
    Swaps spot tokens on Drift.

    Input: A JSON string with:
    {
        "from_symbol": "string, token to swap from",
        "to_symbol": "string, token to swap to",
        "slippage": "float, optional, allowed slippage",
        "to_amount": "float, optional, desired amount of the output token",
        "from_amount": "float, optional, amount of the input token"
    }
    Output:
    {
        "transaction": "dict, swap transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "from_symbol": {"type": str, "required": True},
                "to_symbol": {"type": str, "required": True},
                "slippage": {"type": float, "required": False},
                "to_amount": {"type": float, "required": False},
                "from_amount": {"type": float, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)           
            transaction = await self.solana_kit.drift_swap_spot_token(
                from_symbol=data["from_symbol"],
                to_symbol=data["to_symbol"],
                slippage=data.get("slippage"),
                to_amount=data.get("to_amount"),
                from_amount=data.get("from_amount"),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error swapping spot token on Drift: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")




import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaSNSRegisterDomainTool(BaseTool):
    name: str = "solana_sns_register_domain"
    description: str = """
    Prepares a transaction to register a new SNS domain.

    Input: A JSON string with:
    {
        "domain": "string, the domain to register",
        "buyer": "string, base58 public key of the buyer's wallet",
        "buyer_token_account": "string, base58 public key of the buyer's token account",
        "space": "integer, bytes to allocate for the domain",
        "mint": "string, optional, the token mint public key (default: USDC)",
        "referrer_key": "string, optional, base58 public key of the referrer"
    }

    Output:
    {
        "transaction": "string, base64-encoded transaction object",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "domain": {"type": str, "required": True},
                "buyer": {"type": str, "required": True},
                "buyer_token_account": {"type": str, "required": True},
                "space": {"type": int, "required": True, "min": 1},
                "mint": {"type": str, "required": False},
                "referrer_key": {"type": str, "required": False}
            }
            validate_input(data, schema)

            domain = data["domain"]
            buyer = data["buyer"]
            buyer_token_account = data["buyer_token_account"]
            space = data["space"]
            mint = data.get("mint")
            referrer_key = data.get("referrer_key")

            transaction = await self.solana_kit.get_registration_transaction(
                domain, buyer, buyer_token_account, space, mint, referrer_key
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error preparing registration transaction: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )




import base64
import os

import requests
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes

from agentipy.constants import API_VERSION, BASE_PROXY_URL


def get_encryption_key():
    response = requests.post(f"{BASE_PROXY_URL}/{API_VERSION}/security/get-encryption-key")
    data = response.json()
    return data["requestId"], base64.b64decode(data["encryptionKey"]), base64.b64decode(data["iv"])

def encrypt_private_key(private_key: str):
    """Encrypts the private key using a one-time encryption key from the server."""
    request_id, encryption_key, iv = get_encryption_key()

    cipher = Cipher(algorithms.AES(encryption_key), modes.CBC(iv), backend=default_backend())
    encryptor = cipher.encryptor()

    padding_length = 16 - (len(private_key) % 16)
    padded_private_key = private_key + (chr(padding_length) * padding_length)

    encrypted = encryptor.update(padded_private_key.encode()) + encryptor.finalize()
    
    return {
        "requestId": request_id,
        "encryptedPrivateKey": base64.b64encode(encrypted).decode(),
    }



import requests
from agentipy.agent import SolanaAgentKit

# Send a request to the Block engine url using the JSON RPC methods 
def __send_request(agent: SolanaAgentKit, endpoint, method, params=None):
    if endpoint == None:
        return "Error: Please enter a valid endpoint."

    if agent.jito_uuid == None:
        headers = {
            'Content-Type': 'application/json', 
            "accept": "application/json"
        }
    else:
        headers = {
            'Content-Type': 'application/json', 
            "accept": "application/json",
            "x-jito-atuh": agent.jito_uuid
        }
    data = {
        "id": 1,
        "jsonrpc": "2.0",
        "method": method,
        "params": [params]
    }

    print(data)
    try:
        resp = requests.post(agent.url + endpoint, headers=headers, json=data)
        resp.raise_for_status()
        return {"success": True, "data": resp.json()}
    except requests.exceptions.HTTPError as errh:
        return {"success": False, "error": f"HTTP Error: {errh}"}
    except requests.exceptions.ConnectionError as errc:
        return {"success": False, "error": f"Error Connecting: {errc}"}
    except requests.exceptions.Timeout as errt:
        return {"success": False, "error": f"Timeout Error: {errt}"}
    except requests.exceptions.InvalidHeader as err:
        return {"success": False, "error": f"Invalid Header error: {err}"}
    except requests.exceptions.InvalidURL as err:
        return {"success": False, "error": f"InvalidURL error: {err}"}
    except requests.exceptions.RequestException as err:
        return {"success": False, "error": f"An error occurred: {err}"}



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class Create3LandNFTTool(BaseTool):
    name: str = "create_3land_nft"
    description: str = """
    Creates a 3Land NFT.

    Input: A JSON string with:
    {
        "item_name": "string, name of the NFT",
        "seller_fee": "float, seller fee percentage",
        "item_amount": "int, number of NFTs to mint",
        "item_symbol": "string, symbol of the NFT",
        "item_description": "string, description of the NFT",
        "traits": "Any, NFT traits",
        "price": "float, optional, price of the NFT",
        "main_image_url": "string, optional, URL of the main image",
        "cover_image_url": "string, optional, URL of the cover image",
        "spl_hash": "string, optional, SPL hash identifier",
        "pool_name": "string, optional, pool name",
        "is_devnet": "bool, optional, whether to use devnet",
        "with_pool": "bool, optional, whether to include a pool"
    }
    Output:
    {
        "transaction": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "item_name": {"type": str, "required": True},
                "seller_fee": {"type": float, "required": True},
                "item_amount": {"type": int, "required": True},
                "item_symbol": {"type": str, "required": True},
                "item_description": {"type": str, "required": True},
                "traits": {"type": str, "required": True},
                "price": {"type": float, "required": False},
                "main_image_url": {"type": str, "required": False},
                "cover_image_url": {"type": str, "required": False},
                "spl_hash": {"type": str, "required": False},
                "pool_name": {"type": str, "required": False},
                "is_devnet": {"type": bool, "required": False},
                "with_pool": {"type": bool, "required": False}
            }
            data = json.loads(input)
            validate_input(data, schema)

           
            
            transaction = await self.solana_kit.create_3land_nft(
                item_name=data["item_name"],
                seller_fee=data["seller_fee"],
                item_amount=data["item_amount"],
                item_symbol=data["item_symbol"],
                item_description=data["item_description"],
                traits=data["traits"],
                price=data.get("price"),
                main_image_url=data.get("main_image_url"),
                cover_image_url=data.get("cover_image_url"),
                spl_hash=data.get("spl_hash"),
                pool_name=data.get("pool_name"),
                is_devnet=data.get("is_devnet", False),
                with_pool=data.get("with_pool", False),
            )
            return {
                "transaction": transaction,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction": None,
                "message": f"Error creating 3land NFT: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun.")



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input

class RugCheckFetchTokenFluxLPLockersTool(BaseTool):
    name: str = "rugcheck_fetch_token_flux_lp_lockers"
    description: str = """
    Fetches token flux LP lockers using RugCheckManager.

    Input: A JSON string with:
    {
        "token_id": "string, the ID of the token"
    }
    Output:
    {
        "flux_lp_lockers": "list, a list of token flux LP lockers",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "token_id": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
            flux_lp_lockers = await self.agent_kit.fetch_token_flux_lp_lockers(
                token_id=data["token_id"]
            )
            return {
                "flux_lp_lockers": flux_lp_lockers,
                "message": "Success"
            }
        except Exception as e:
            return {
                "flux_lp_lockers": None,
                "message": f"Error fetching token flux LP lockers: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json
from agentipy.agent import SolanaAgentKit
from langchain.tools import BaseTool
from agentipy.helpers import validate_input

class BackpackGetOpenPositionsTool(BaseTool):
    name: str = "backpack_get_open_positions"
    description: str = """
    Fetches open positions using the BackpackManager.

    Input: None
    Output:
    {
        "open_positions": "list, the open positions",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            open_positions = await self.solana_kit.get_open_positions()
            return {
                "open_positions": open_positions,
                "message": "Success"
            }
        except Exception as e:
            return {
                "open_positions": None,
                "message": f"Error fetching open positions: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class BackpackGetBorrowLendPositionsTool(BaseTool):
    name: str = "backpack_get_borrow_lend_positions"
    description: str = """
    Fetches borrow/lend positions using the BackpackManager.

    Input: None
    Output:
    {
        "positions": "list, the borrow/lend positions",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            positions = await self.solana_kit.get_borrow_lend_positions()
            return {
                "positions": positions,
                "message": "Success"
            }
        except Exception as e:
            return {
                "positions": None,
                "message": f"Error fetching borrow/lend positions: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class BackpackExecuteBorrowLendTool(BaseTool):
    name: str = "backpack_execute_borrow_lend"
    description: str = """
    Executes a borrow/lend operation using the BackpackManager.

    Input: A JSON string with:
    {
        "quantity": "string, the amount to borrow or lend",
        "side": "string, either 'borrow' or 'lend'",
        "symbol": "string, the token symbol"
    }
    Output:
    {
        "result": "dict, the result of the operation",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "quantity": {"type": str, "required": True},
                "side": {"type": str, "required": True},
                "symbol": {"type": str, "required": True}
            }
            validate_input(data, schema)

            result = await self.solana_kit.execute_borrow_lend(
                quantity=data["quantity"],
                side=data["side"],
                symbol=data["symbol"]
            )
            return {
                "result": result,
                "message": "Success"
            }
        except Exception as e:
            return {
                "result": None,
                "message": f"Error executing borrow/lend operation: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class BackpackGetBorrowHistoryTool(BaseTool):
    name: str = "backpack_get_borrow_history"
    description: str = """
    Fetches borrow history using the BackpackManager.

    Input: A JSON string with optional filters for the query.
    Output:
    {
        "borrow_history": "list, the borrow history records",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            borrow_history = await self.solana_kit.get_borrow_history(**data)
            return {
                "borrow_history": borrow_history,
                "message": "Success"
            }
        except Exception as e:
            return {
                "borrow_history": None,
                "message": f"Error fetching borrow history: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class BackpackGetInterestHistoryTool(BaseTool):
    name: str = "backpack_get_interest_history"
    description: str = """
    Fetches interest history using the BackpackManager.

    Input: A JSON string with optional filters for the query.
    Output:
    {
        "interest_history": "list, the interest history records",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            interest_history = await self.solana_kit.get_interest_history(**data)
            return {
                "interest_history": interest_history,
                "message": "Success"
            }
        except Exception as e:
            return {
                "interest_history": None,
                "message": f"Error fetching interest history: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaHeliusCreateWebhookTool(BaseTool):
    name: str = "solana_helius_create_webhook"
    description: str = """
    Create a webhook for transaction events.

    Input: A JSON string with:
    {
        "webhook_url": "URL to send the webhook data",
        "transaction_types": "List of transaction types to listen for",
        "account_addresses": "List of account addresses to monitor",
        "webhook_type": "Type of webhook",
        "txn_status": "optional, transaction status to filter by",
        "auth_header": "optional, authentication header for the webhook"
    }

    Output:
    {
        "status": "success",
        "data": "Webhook creation response"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "webhook_url": {"type": str, "required": True},
                "transaction_types": {"type": list, "required": True},
                "account_addresses": {"type": list, "required": True},
                "webhook_type": {"type": str, "required": True},
                "auth_header": {"type": str, "required": False}
            }
            validate_input(data, schema)

            webhook_url = data["webhook_url"]
            transaction_types = data["transaction_types"]
            account_addresses = data["account_addresses"]
            webhook_type = data["webhook_type"]
            txn_status = data.get("txn_status", "all")
            auth_header = data.get("auth_header", None)

            result = await self.solana_kit.create_webhook(
                webhook_url, transaction_types, account_addresses, webhook_type, txn_status, auth_header
            )
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )

class SolanaHeliusGetAllWebhooksTool(BaseTool):
    name: str = "solana_helius_get_all_webhooks"
    description: str = """
    Fetch all webhooks created in the system.

    Input: None (No parameters required)

    Output:
    {
        "status": "success",
        "data": "List of all webhooks"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self):
        try:
            result = await self.solana_kit.get_all_webhooks()
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )

class SolanaHeliusGetWebhookTool(BaseTool):
    name: str = "solana_helius_get_webhook"
    description: str = """
    Retrieve a specific webhook by ID.

    Input: A JSON string with:
    {
        "webhook_id": "ID of the webhook to retrieve"
    }

    Output:
    {
        "status": "success",
        "data": "Webhook details"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "webhook_id": {"type": str, "required": True}
            }
            validate_input(data, schema)
            webhook_id = data["webhook_id"]
            result = await self.solana_kit.get_webhook(webhook_id)
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )
class SolanaHeliusEditWebhookTool(BaseTool):
    name: str = "solana_helius_edit_webhook"
    description: str = """
    Edit an existing webhook by its ID.

    Input: A JSON string with:
    {
        "webhook_id": "ID of the webhook to edit",
        "webhook_url": "Updated URL for the webhook",
        "transaction_types": "Updated list of transaction types",
        "account_addresses": "Updated list of account addresses",
        "webhook_type": "Updated webhook type",
        "txn_status": "optional, updated transaction status filter",
        "auth_header": "optional, updated authentication header"
    }

    Output:
    {
        "status": "success",
        "data": "Updated webhook details"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "webhook_id": {"type": str, "required": True},
                "webhook_url": {"type": str, "required": True},
                "transaction_types": {"type": list, "required": True},
                "account_addresses": {"type": list, "required": True},
                "webhook_type": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
         
            webhook_id = data["webhook_id"]
            webhook_url = data["webhook_url"]
            transaction_types = data["transaction_types"]
            account_addresses = data["account_addresses"]
            webhook_type = data["webhook_type"]
            txn_status = data.get("txn_status", "all")
            auth_header = data.get("auth_header", None)

            result = await self.solana_kit.edit_webhook(
                webhook_id, webhook_url, transaction_types, account_addresses, webhook_type, txn_status, auth_header
            )
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )

class SolanaHeliusDeleteWebhookTool(BaseTool):
    name: str = "solana_helius_delete_webhook"
    description: str = """
    Delete a webhook by its ID.

    Input: A JSON string with:
    {
        "webhook_id": "ID of the webhook to delete"
    }

    Output:
    {
        "status": "success",
        "data": "Webhook deletion confirmation"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "webhook_id": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
            webhook_id = data["webhook_id"] 
            result = await self.solana_kit.delete_webhook(webhook_id)
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )




import logging
from typing import Any, Dict, Optional

import requests

from agentipy.agent import SolanaAgentKit
from agentipy.utils.agentipy_proxy.utils import encrypt_private_key

logger = logging.getLogger(__name__)


class SquadsManager:
    """
    Manager class to handle Squads Protocol functionalities such as approving proposals,
    creating multisigs, creating proposals, and depositing to treasury.
    """

    @staticmethod
    def approve_multisig_proposal(
        agent: SolanaAgentKit,
        transaction_index: int,
    ) -> Optional[Dict[str, Any]]:
        """
        Approves a multisig proposal.

        :param agent: An instance of SolanaAgentKit.
        :param transaction_index: The index of the transaction to approve.
        :return: Transaction signature or error details.
        """
        try:
            if transaction_index < 0:
                raise ValueError("Invalid transaction index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "transactionIndex": transaction_index,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-approve-proposal",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during multisig proposal approval: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during multisig proposal approval: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def create_squads_multisig(
        agent: SolanaAgentKit,
        creator: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Creates a Squads multisig.

        :param agent: An instance of SolanaAgentKit.
        :param creator: The public key of the multisig creator.
        :return: Transaction signature or error details.
        """
        try:

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "creator": creator,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/create-squads-multisig",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except ValueError:
            logger.error("Invalid creator public key.", exc_info=True)
            return {"success": False, "error": "Invalid creator public key."}
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during multisig creation: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during multisig creation: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def create_multisig_proposal(
        agent: SolanaAgentKit,
        transaction_index: int,
    ) -> Optional[Dict[str, Any]]:
        """
        Creates a multisig proposal.

        :param agent: An instance of SolanaAgentKit.
        :param transaction_index: The index of the transaction to create a proposal for.
        :return: Transaction signature or error details.
        """
        try:
            if transaction_index < 0:
                raise ValueError("Invalid transaction index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "transactionIndex": transaction_index,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-create-proposal",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during multisig proposal creation: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during multisig proposal creation: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def deposit_to_multisig_treasury(
        agent: SolanaAgentKit,
        amount: float,
        vault_index: int,
        mint: Optional[str] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Deposits funds into the multisig treasury.

        :param agent: An instance of SolanaAgentKit.
        :param amount: The amount to deposit.
        :param vault_index: The vault index.
        :param mint: The mint address (optional).
        :return: Transaction signature or error details.
        """
        try:
            if vault_index < 0:
                raise ValueError("Invalid vault index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "vaultIndex": vault_index,
                "mint": mint if mint else None,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-deposit-to-treasury",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except ValueError:
            logger.error("Invalid vault index or mint public key.", exc_info=True)
            return {"success": False, "error": "Invalid vault index or mint public key."}
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during deposit to multisig treasury: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during deposit to multisig treasury: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def execute_multisig_proposal(
        agent: SolanaAgentKit,
        transaction_index: int,
    ) -> Optional[Dict[str, Any]]:
        """
        Executes a multisig proposal.

        :param agent: An instance of SolanaAgentKit.
        :param transaction_index: The index of the transaction to execute.
        :return: Transaction signature or error details.
        """
        try:
            if transaction_index < 0:
                raise ValueError("Invalid transaction index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "transactionIndex": transaction_index,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-execute-proposal",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during multisig proposal execution: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during multisig proposal execution: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def reject_multisig_proposal(
        agent: SolanaAgentKit,
        transaction_index: int,
    ) -> Optional[Dict[str, Any]]:
        """
        Rejects a multisig proposal.

        :param agent: An instance of SolanaAgentKit.
        :param transaction_index: The index of the transaction to reject.
        :return: Transaction signature or error details.
        """
        try:
            if transaction_index < 0:
                raise ValueError("Invalid transaction index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "transactionIndex": transaction_index,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-reject-proposal",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during multisig proposal rejection: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during multisig proposal rejection: {error}", exc_info=True)
            return {"success": False, "error": str(error)}

    @staticmethod
    def transfer_from_multisig_treasury(
        agent: SolanaAgentKit,
        amount: float,
        to: str,
        vault_index: int,
        mint: str,
    ) -> Optional[Dict[str, Any]]:
        """
        Transfers assets from the multisig treasury.

        :param agent: An instance of SolanaAgentKit.
        :param amount: The amount to transfer.
        :param to: The recipient address.
        :param vault_index: The index of the vault.
        :param mint: The mint address of the token to transfer.
        :return: Transaction signature or error details.
        """
        try:
            if vault_index < 0:
                raise ValueError("Invalid vault index.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "to": to,
                "vaultIndex": vault_index,
                "mint": mint,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/squads/multisig-transfer-from-treasury",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except ValueError:
            logger.error("Invalid vault index or mint public key.", exc_info=True)
            return {"success": False, "error": "Invalid vault index or mint public key."}
        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during transfer from multisig treasury: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during transfer from multisig treasury: {error}", exc_info=True)
            return {"success": False, "error": str(error)}


import json

from langchain.tools import BaseTool

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaCalculatePumpCurvePriceTool(BaseTool):
    name: str = "solana_calculate_pump_curve_price"
    description: str = """
    Calculate the price for a bonding curve based on its state.

    Input: A JSON string with:
    {
        "curve_state": "BondingCurveState object as a dictionary"
    }

    Output:
    {
        "status": "success",
        "price": "The calculated price"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "curve_state": {"type": str, "required": True}
            }
            validate_input(data, schema)

            curve_state = data["curve_state"]

            result = await self.solana_kit.calculate_pump_curve_price(curve_state)
            return {
                "status": "success",
                "price": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution.")



import logging
from typing import Any, Dict, Optional

import requests

from agentipy.agent import SolanaAgentKit
from agentipy.utils.agentipy_proxy.utils import encrypt_private_key

logger = logging.getLogger(__name__)


class TiplinkManager:
    """
    Manager class to handle Tiplink functionalities such as creating Tiplinks.
    """

    @staticmethod
    def create_tiplink(
        agent: SolanaAgentKit,
        amount: float,
        spl_mint_address: Optional[str] = None,
    ) -> Optional[Dict[str, Any]]:
        """
        Creates a Tiplink.

        :param agent: An instance of SolanaAgentKit.
        :param amount: The amount to fund the Tiplink with.
        :param spl_mint_address: (Optional) SPL token mint address.
        :return: Transaction signature and URL or error details.
        """
        try:
            if amount <= 0:
                raise ValueError("Amount must be greater than zero.")

            encrypted_private_key = encrypt_private_key(agent.private_key)

            payload = {
                "requestId": encrypted_private_key["requestId"],
                "encrypted_private_key": encrypted_private_key["encryptedPrivateKey"],
                "rpc_url": agent.rpc_url,
                "open_api_key": agent.openai_api_key,
                "amount": amount,
                "splMintAddress": spl_mint_address if spl_mint_address else None,
            }

            response = requests.post(
                f"{agent.base_proxy_url}/{agent.api_version}/tiplink/create-tiplink",
                json=payload,
                headers={"Content-Type": "application/json"},
            )
            response.raise_for_status()

            data = response.json()
            return data if data.get("success") else {"success": False, "error": data.get("error", "Unknown error")}

        except requests.exceptions.RequestException as http_error:
            logger.error(f"HTTP error during Tiplink creation: {http_error}", exc_info=True)
            return {"success": False, "error": str(http_error)}
        except Exception as error:
            logger.error(f"Unexpected error during Tiplink creation: {error}", exc_info=True)
            return {"success": False, "error": str(error)}



from agentipy.agent import SolanaAgentKit
from agentipy.langchain.virtuals.prototype import (VirtualsBuyPrototypeTool,
                                                   VirtualsSellPrototypeTool)
from agentipy.langchain.virtuals.sentient import (
    VirtualsBuySentientTool, VirtualsGetSentientListingsTool,
    VirtualsSellSentientTool)


def get_solana_virtuals_tools(solana_kit: SolanaAgentKit):
    return [
        VirtualsBuyPrototypeTool(solana_kit=solana_kit),
        VirtualsSellPrototypeTool(solana_kit=solana_kit),
        VirtualsBuySentientTool(solana_kit=solana_kit),
        VirtualsGetSentientListingsTool(solana_kit=solana_kit),
        VirtualsSellSentientTool(solana_kit=solana_kit),
    ]


from typing import List

from agentipy.agent import SolanaAgentKit
from agentipy.utils.helius.helpers.utility import (_make_delete_request,
                                                   _make_get_request,
                                                   _make_post_request,
                                                   _make_put_request)


class HeliusManager:
    def get_balances(agent: SolanaAgentKit, address: str):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/addresses/{address}/balances"
        url = base_url + path + api_key_query
        return _make_get_request(url)

    def get_address_name(agent: SolanaAgentKit, address: str):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/addresses/{address}/names"
        url = base_url + path + api_key_query
        return _make_get_request(url)

    def get_nft_events(agent: SolanaAgentKit, accounts: List[str], types: List[str] = None, sources: List[str] = None, start_slot: int = None, end_slot: int = None, start_time: int = None, end_time: int = None, first_verified_creator: List[str] = None, verified_collection_address: List[str] = None, limit: int = None, sort_order: str = None, pagination_token: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v1/nft-events"
        url = base_url + path + api_key_query
        payload = {
            "query": {
                "accounts": accounts,
                "types": types,
                "sources": sources,
                "startSlot": start_slot,
                "endSlot": end_slot,
                "startTime": start_time,
                "endTime": end_time,
                "nftCollectionFilters": {
                    "firstVerifiedCreator": first_verified_creator,
                    "verifiedCollectionAddress": verified_collection_address
                }
            },
            "options": {
                "limit": limit,
                "sortOrder": sort_order,
                "paginationToken": pagination_token
            }
        }
        return _make_post_request(url, payload)

    def get_mintlists(agent: SolanaAgentKit, first_verified_creators: List[str], verified_collection_addresses: List[str] = None, limit: int = None, pagination_token: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v1/mintlist"
        url = base_url + path + api_key_query
        payload = {
            "query": {
                "firstVerifiedCreators": first_verified_creators,
                "verifiedCollectionAddresses": verified_collection_addresses
            },
            "options": {
                "limit": limit,
                "paginationToken": pagination_token
            }
        }
        return _make_post_request(url, payload)

    def get_nft_fingerprint(agent: SolanaAgentKit, mints: List[str]):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v1/nfts"
        url = base_url + path + api_key_query
        payload = {
            "mints": mints
        }
        return _make_post_request(url, payload)

    def get_active_listings(agent: SolanaAgentKit, first_verified_creators: List[str], verified_collection_addresses: List[str] = None, marketplaces: List[str] = None, limit: int = None, pagination_token: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v1/active-listings"
        url = base_url + path + api_key_query
        payload = {
            "query": {
                "marketplaces": marketplaces,
                "firstVerifiedCreators": first_verified_creators,
                "verifiedCollectionAddresses": verified_collection_addresses
            },
            "options": {
                "limit": limit,
                "paginationToken": pagination_token
            }
        }
        return _make_post_request(url, payload)

    def get_nft_metadata(agent: SolanaAgentKit, mint_accounts: List[str]):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v0/tokens/metadata"
        url = base_url + path + api_key_query
        payload = {
            "mintAccounts": mint_accounts
        }
        return _make_post_request(url, payload)

    def get_raw_transactions(agent: SolanaAgentKit, accounts: List[str], start_slot: int = None, end_slot: int = None, start_time: int = None, end_time: int = None, limit: int = None, sort_order: str = None, pagination_token: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v1/raw-transactions"
        url = base_url + path + api_key_query
        payload = {
            "query": {
                "accounts": accounts,
                "startSlot": start_slot,
                "endSlot": end_slot,
                "startTime": start_time,
                "endTime": end_time
            },
            "options": {
                "limit": limit,
                "sortOrder": sort_order,
                "paginationToken": pagination_token
            }
        }
        return _make_post_request(url, payload)

    def get_parsed_transactions(agent: SolanaAgentKit, transactions: List[str], commitment: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v0/transactions"
        if commitment:
            url = base_url + path + api_key_query + "?commitment=" + commitment
        else:
            url = base_url + path + api_key_query
        payload = {
            "transactions": transactions
        }
        return _make_post_request(url, payload)

    def get_parsed_transaction_history(agent: SolanaAgentKit, address: str, before: str = '', until: str = '', commitment: str = '', source: str = '', type: str = ''):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/addresses/{address}/transactions"
        params = {
            "before": before,
            "until": until,
            "commitment": commitment,
            "source": source,
            "type": type
        }
        url = base_url + path + api_key_query
        return _make_get_request(url, params=params)

    def create_webhook(agent: SolanaAgentKit, webhook_url: str, transaction_types: list, account_addresses: list, webhook_type: str, txn_status: str = "all", auth_header: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v0/webhooks"
        url = base_url + path + api_key_query
        payload = {
            "webhookURL": webhook_url,
            "transactionTypes": transaction_types,
            "accountAddresses": account_addresses,
            "webhookType": webhook_type,
            "txnStatus": txn_status
        }
        if auth_header:
            payload["authHeader"] = auth_header
        return _make_post_request(url, payload)

    def get_all_webhooks(agent: SolanaAgentKit):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = "/v0/webhooks"
        url = base_url + path + api_key_query
        return _make_get_request(url)

    def get_webhook(agent: SolanaAgentKit, webhook_id: str):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/webhooks/{webhook_id}"
        url = base_url + path + api_key_query
        return _make_get_request(url)

    def edit_webhook(agent: SolanaAgentKit, webhook_id: str, webhook_url: str, transaction_types: list, account_addresses: list, webhook_type: str, txn_status: str = "all", auth_header: str = None):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/webhooks/{webhook_id}"
        url = base_url + path + api_key_query
        payload = {
            "webhookURL": webhook_url,
            "transactionTypes": transaction_types,
            "accountAddresses": account_addresses,
            "webhookType": webhook_type,
            "txnStatus": txn_status
        }
        if auth_header:
            payload["authHeader"] = auth_header
        return _make_put_request(url, payload)

    def delete_webhook(agent: SolanaAgentKit, webhook_id: str):
        base_url = agent.helius_rpc_url
        api_key_query = f"?api-key={agent.helius_api_key}"
        path = f"/v0/webhooks/{webhook_id}"
        url = base_url + path + api_key_query
        return _make_delete_request(url)



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input

class SolanaHeliusGetParsedTransactionHistoryTool(BaseTool):
    name: str = "solana_helius_get_parsed_transaction_history"
    description: str = """
    Fetch parsed transaction history for a given address.

    Input: A JSON string with:
    {
        "address": "string, the account address",
        "before": "optional before transaction timestamp",
        "until": "optional until transaction timestamp",
        "commitment": "optional commitment level",
        "source": "optional source of transaction",
        "type": "optional type of transaction"
    }

    Output:
    {
        "transaction_history": "list of parsed transaction history"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "address": {"type": str, "required": True},
                "before": {"type": str, "required": False},
                "until": {"type": str, "required": False},
                "commitment": {"type": str, "required": False},
                "source": {"type": str, "required": False},
                "type": {"type": str, "required": False}
            }
            validate_input(data, schema)

            address = data["address"]
            before = data.get("before", "")
            until = data.get("until", "")
            commitment = data.get("commitment", "")
            source = data.get("source", "")
            type = data.get("type", "")

            result = await self.solana_kit.get_parsed_transaction_history(
                address, before, until, commitment, source, type
            )
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaHeliusGetNFTFingerprintTool(BaseTool):
    name: str = "solana_helius_get_nft_fingerprint"
    description: str = """
    Fetch NFT fingerprint for a list of mint addresses.

    Input: A JSON string with:
    {
        "mints": ["string, the mint addresses of the NFTs"]
    }

    Output:
    {
        "fingerprint": "list of NFT fingerprint data"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:    
            data = json.loads(input)
            schema = {
                "mints": {"type": list, "required": True}
            }
            validate_input(data, schema)
            mints = data["mints"]

            result = await self.solana_kit.get_nft_fingerprint(mints)
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        """Synchronous version of the run method, required by BaseTool."""
        raise NotImplementedError(
            "This tool only supports async execution via _arun. Please use the async interface."
        )



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input

class RugCheckFetchDomainRecordsTool(BaseTool):
    name: str = "rugcheck_fetch_domain_records"
    description: str = """
    Fetches all records for a domain using RugCheckManager.

    Input: A JSON string with:
    {
        "domain": "string, the domain name"
    }
    Output:
    {
        "records": "list, a list of all records for the domain",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            schema = {
                "domain": {"type": str, "required": True}
            }
            data = json.loads(input)
            validate_input(data, schema)
            records = await self.agent_kit.fetch_domain_records(
                domain=data["domain"]
            )
            return {
                "records": records,
                "message": "Success"
            }
        except Exception as e:
            return {
                "records": None,
                "message": f"Error fetching domain records: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json

from langchain.tools import BaseTool

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class VirtualsBuyPrototypeTool(BaseTool):
    name: str = "virtuals_buy_prototype"
    description: str = """
    Purchases Prototype tokens using VirtualsManager.

    Input: A JSON string with:
    {
        "token_address": "string, the token address",
        "amount": "string, the amount to purchase",
        "builder_id": "int, optional, the builder ID for the purchase",
        "slippage": "float, optional, the slippage tolerance percentage"
    }
    Output:
    {
        "transaction_receipt": "dict, the transaction receipt",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "token_address": str,
                "amount": str,
                "builder_id": int
            }
            validate_input(data, schema)
            transaction_receipt = await self.agent_kit.buy_prototype(
                token_address=data["token_address"],
                amount=data["amount"],
                builder_id=data.get("builder_id"),
                slippage=data.get("slippage")
            )
            return {
                "transaction_receipt": transaction_receipt,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction_receipt": None,
                "message": f"Error purchasing Prototype tokens: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")

class VirtualsSellPrototypeTool(BaseTool):
    name: str = "virtuals_sell_prototype"
    description: str = """
    Sells Prototype tokens using VirtualsManager.

    Input: A JSON string with:
    {
        "token_address": "string, the token address",
        "amount": "string, the amount to sell",
        "builder_id": "int, optional, the builder ID for the sale",
        "slippage": "float, optional, the slippage tolerance percentage"
    }
    Output:
    {
        "transaction_receipt": "dict, the transaction receipt",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "token_address": str,
                "amount": str,
                "builder_id": int
            }
            
            transaction_receipt = await self.agent_kit.sell_prototype(
                token_address=data["token_address"],
                amount=data["amount"],
                builder_id=data.get("builder_id"),
                slippage=data.get("slippage")
            )
            return {
                "transaction_receipt": transaction_receipt,
                "message": "Success"
            }
        except Exception as e:
            return {
                "transaction_receipt": None,
                "message": f"Error selling Prototype tokens: {str(e)}"
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json

from langchain.tools import BaseTool

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class VoltrGetPositionValuesTool(BaseTool):
    name: str = "voltr_get_position_values"
    description: str = """
    Retrieves position values for a given vault in Voltr.

    Input: A JSON string with:
    {
        "vault": "string, the vault address"
    }

    Output:
    {
        "position_values": "dict, position values",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "vault": {"type": str, "required": True},
            }
            validate_input(data, schema)

            vault = data["vault"]

            result = await self.solana_kit.get_position_values(vault)
            return {
                "status": "success",
                "position_values": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")


import logging
from typing import Any, Dict, Optional

import requests

from agentipy.agent import SolanaAgentKit

logger = logging.getLogger(__name__)

class NameServiceManager:
    @staticmethod
    def resolve_name_to_address(agent: SolanaAgentKit, domain: str) -> Optional[str]:
        """
        Resolves a Solana Name Service domain to its corresponding address using QuickNode RPC.

        :param domain: The SNS domain (e.g., example.sol).
        :return: The associated Solana address or None if not found.
        """
        try:
            if not domain:
                raise ValueError("Domain name is required")

            payload = {
                "id": 42,
                "jsonrpc": "2.0",
                "method": "sns_resolveDomain",
                "params": [domain]
            }

            response = requests.post(
                agent.quicknode_rpc_url,
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            return data.get("result")
        except Exception as error:
            logger.error(f"Error resolving domain to address: {str(error)}", exc_info=True)
            return None
        
    @staticmethod
    def get_favourite_domain(agent: SolanaAgentKit, owner: str) -> Optional[str]:
        """
        Resolves a Solana Name Service domain to its corresponding address using QuickNode RPC.

        :param domain: The SNS domain (e.g., example.sol).
        :return: The associated Solana address or None if not found.
        """
        try:
            if not owner:
                raise ValueError("Domain name is required")

            payload = {
                "id": 42,
                "jsonrpc": "2.0",
                "method": "sns_getFavouriteDomain",
                "params": [owner]
            }

            response = requests.post(
                agent.quicknode_rpc_url,
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            return data.get("result")
        except Exception as error:
            logger.error(f"Error resolving domain to address: {str(error)}", exc_info=True)
            return None
        
    @staticmethod
    def get_all_domains_for_owner(agent: SolanaAgentKit, owner: str) -> Optional[str]:
        """
        Resolves a Solana Name Service domain to its corresponding address using QuickNode RPC.

        :param domain: The SNS domain (e.g., example.sol).
        :return: The associated Solana address or None if not found.
        """
        try:
            if not owner:
                raise ValueError("Domain name is required")

            payload = {
                "id": 42,
                "jsonrpc": "2.0",
                "method": "sns_getAllDomainsForOwner",
                "params": [owner]
            }

            response = requests.post(
                agent.quicknode_rpc_url,
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            return data.get("result")
        except Exception as error:
            logger.error(f"Error resolving domain to address: {str(error)}", exc_info=True)
            return None
        
    @staticmethod
    def get_registration_transaction(agent:SolanaAgentKit, domain: str, buyer: str, buyer_token_account: str, space: int, 
                                     mint: Optional[str] = None, referrer_key: Optional[str] = None) -> Optional[str]:
        """
        Returns a ready-to-sign, base64-encoded transaction object to register a new SNS domain.

        :param domain: The domain name to register.
        :param buyer: The base58-encoded Solana public key of the buyer's paying wallet.
        :param buyer_token_account: The base58-encoded Solana public key of the buyer's paying token account.
        :param space: The number of bytes to allocate in the new registered domain.
        :param mint: (Optional) The Solana public key of the Token mint used for payment, defaults to USDC.
        :param referrer_key: (Optional) The base58-encoded Solana public key of the registration referrer.
        :return: The base64-encoded Solana transaction object or None if an error occurs.
        """
        try:
            if not all([domain, buyer, buyer_token_account, space]):
                raise ValueError("Domain, buyer, buyer_token_account, and space are required")

            payload: Dict[str, Any] = {
                "id": 1,
                "jsonrpc": "2.0",
                "method": "sns_getRegistrationTransaction",
                "params": [
                    domain,
                    buyer,
                    buyer_token_account,
                    space
                ]
            }

            if mint:
                payload["params"].append(mint)
            if referrer_key:
                payload["params"].append(referrer_key)

            response = requests.post(
                agent.quicknode_rpc_url,
                json=payload,
                headers={"Content-Type": "application/json"}
            )
            response.raise_for_status()

            data = response.json()
            return data.get("result")
        except Exception as error:
            logger.error(f"Error getting registration transaction: {str(error)}", exc_info=True)
            return None


import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input

class SolanaHeliusGetMintlistsTool(BaseTool):
    name: str = "solana_helius_get_mintlists"
    description: str = """
    Fetch mintlists for a given list of verified creators.

    Input: A JSON string with:
    {
        "first_verified_creators": "List of first verified creator addresses",
        "verified_collection_addresses": "Optional list of verified collection addresses",
        "limit": "Optional limit for results",
        "pagination_token": "Optional pagination token"
    }

    Output: {
        "mintlists": List[dict], # list of mintlists matching the criteria
        "status": "success" or "error",
        "message": "Error message if any"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "first_verified_creators": {"type": list, "required": True},
                "verified_collection_addresses": {"type": list, "required": False},
                "limit": {"type": int, "required": False},
                "pagination_token": {"type": str, "required": False}
            }
            validate_input(data, schema)

            result = await self.solana_kit.get_mintlists(
                first_verified_creators=data["first_verified_creators"],
                verified_collection_addresses=data.get("verified_collection_addresses"),
                limit=data.get("limit"),
                pagination_token=data.get("pagination_token")
            )
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class ElfaAiPingApiTool(BaseTool):
    name: str = "elfa_ai_ping_api"
    description: str = """
    Pings the Elfa AI API using ElfaAiManager.

    Input: None
    Output:
    {
        "api_response": "dict, the response from Elfa AI API",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str = None): 
        try:
            api_response = await self.agent_kit.ping_elfa_ai_api()
            return {
                "api_response": api_response,
                "message": "Success"
            }
        except Exception as e:
            return {
                "api_response": None,
                "message": f"Error pinging Elfa AI API: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")


class ElfaAiGetApiKeyStatusTool(BaseTool):
    name: str = "elfa_ai_get_api_key_status"
    description: str = """
    Retrieves the status of the Elfa AI API key using ElfaAiManager.

    Input: None
    Output:
    {
        "api_key_status": "dict, the status of the API key",
        "message": "string, if an error occurs"
    }
    """
    agent_kit: SolanaAgentKit

    async def _arun(self, input: str = None): 
        try:
            api_key_status = await self.agent_kit.get_elfa_ai_api_key_status()
            return {
                "api_key_status": api_key_status,
                "message": "Success"
            }
        except Exception as e:
            return {
                "api_key_status": None,
                "message": f"Error fetching Elfa AI API key status: {str(e)}"
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")


import json
from langchain.tools import BaseTool
from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaHeliusGetBalancesTool(BaseTool):
    name: str = "solana_helius_get_balances"
    description: str = """
    Fetch the balances for a given Solana address.

    Input: A JSON string with:
    {
        "address": "string, the Solana address"
    }

    Output: {
        "balances": List[dict], # the list of token balances for the address
        "status": "success" or "error",
        "message": "Error message if any"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "address": {"type": str, "required": True}
            }
            validate_input(data, schema)

            address = data["address"]

            result = await self.solana_kit.get_balances(address)
            return {
                "status": "success",
                "data": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")



from agentipy.agent import SolanaAgentKit
from agentipy.langchain.faucet.solana_request_funds import SolanaFaucetTool


def get_faucet_tools(solana_kit: SolanaAgentKit):
    return [
        SolanaFaucetTool(solana_kit=solana_kit),
    ]  


from mcp.types import Tool

from agentipy.tools.use_debridge import DeBridgeManager

DEBRIDGE_ACTIONS = {
    "CREATE_DEBRIDGE_TRANSACTION": Tool(
        name="CREATE_DEBRIDGE_TRANSACTION",
        description=(
            "Create a cross-chain bridge transaction using deBridge Liquidity Network API. "
            "input_schema Example: { src_chain_id: string, src_chain_token_in: string, src_chain_token_in_amount: string, "
            "dst_chain_id: string, dst_chain_token_out: string, dst_chain_token_out_recipient: string, "
            "src_chain_order_authority_address: string, dst_chain_order_authority_address: string, "
            "affiliate_fee_percent: string = '0', affiliate_fee_recipient: string|undefined = undefined, prepend_operating_expenses: boolean = true, "
            "dst_chain_token_out_amount: string = 'auto' }"
        ),
        inputSchema={
            "src_chain_id": {"type": "string", "description": "The internal chain ID of the source chain."},
            "src_chain_token_in": {"type": "string", "description": "The address of the input token (token being sold)."},
            "src_chain_token_in_amount": {
                "type": "string",
                "description": "The amount of input token to sell (with decimals), or 'auto'.",
            },
            "dst_chain_id": {"type": "string", "description": "The internal chain ID of the destination chain."},
            "dst_chain_token_out": {"type": "string", "description": "The address of the output token (token being bought)."},
            "dst_chain_token_out_recipient": {"type": "string", "description": "The recipient address on the destination chain."},
            "src_chain_order_authority_address": {"type": "string", "description": "The address on the source chain for order authority."},
            "dst_chain_order_authority_address": {"type": "string", "description": "The address on the destination chain for order authority."},
            "affiliate_fee_percent": {"type": "string", "description": "The percentage of affiliate fee to cut off. Defaults to '0'."},
            "affiliate_fee_recipient": {"type": "string", "description": "The recipient address of the affiliate fee. Optional."},
            "prepend_operating_expenses": {"type": "boolean", "description": "Whether to prepend operating expenses. Defaults to True."},
            "dst_chain_token_out_amount": {
                "type": "string",
                "description": "The amount of output token to buy (with decimals), or 'auto'. Defaults to 'auto'.",
            },
        },
        handler=lambda agent, params: DeBridgeManager.create_debridge_transaction(
            agent,
            src_chain_id=params["src_chain_id"],
            src_chain_token_in=params["src_chain_token_in"],
            src_chain_token_in_amount=params["src_chain_token_in_amount"],
            dst_chain_id=params["dst_chain_id"],
            dst_chain_token_out=params["dst_chain_token_out"],
            dst_chain_token_out_recipient=params["dst_chain_token_out_recipient"],
            src_chain_order_authority_address=params["src_chain_order_authority_address"],
            dst_chain_order_authority_address=params["dst_chain_order_authority_address"],
            affiliate_fee_percent=params["affiliate_fee_percent"],
            affiliate_fee_recipient=params["affiliate_fee_recipient"],
            prepend_operating_expenses=params["prepend_operating_expenses"],
            dst_chain_token_out_amount=params["dst_chain_token_out_amount"],
        ),
    ),

    "EXECUTE_DEBRIDGE_TRANSACTION": Tool(
        name="EXECUTE_DEBRIDGE_TRANSACTION",
        description="Execute a cross-chain bridge transaction using deBridge Liquidity Network API. input_schema Example: { transaction_data: { data: string } }",
        inputSchema={
            "transaction_data": {
                "type": "object",
                "description": "The transaction object returned by the bridge API. Must contain a 'data' field (base64).",
                "properties": {
                    "data": {
                        "type": "string",
                        "description": "Base64-encoded transaction to execute.",
                    }
                },
                "required": ["data"],
            },
        },
        handler=lambda agent, params: DeBridgeManager.execute_debridge_transaction(
            agent,
            transaction_data=params["transaction_data"],
        ),
    ),

    "CHECK_TRANSACTION_STATUS": Tool(
        name="CHECK_TRANSACTION_STATUS",
        description="Check the status of a cross-chain bridge transaction using deBridge Liquidity Network API. input_schema Example: { tx_hash: string }",
        inputSchema={
            "tx_hash": {
                "type": "string",
                "description": "The transaction hash to check the status for.",
            },
        },
        handler=lambda agent, params: DeBridgeManager.check_transaction_status(
            agent,
            tx_hash=params["tx_hash"],
        ),
    ),
}



import json

from langchain.tools import BaseTool
from solders.pubkey import Pubkey  # type: ignore

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class SolanaBuyPumpfunTokenTool(BaseTool):
    name: str = "solana_buy_token"
    description: str = """
    Buy a specific amount of tokens using the bonding curve.

    Input: A JSON string with:
    {
        "mint": "The mint address of the token as a string",
        "bonding_curve": "The bonding curve public key as a string",
        "associated_bonding_curve": "The associated bonding curve public key as a string",
        "amount": "The amount of tokens to buy",
        "slippage": "The allowed slippage percentage",
        "max_retries": "Maximum retries for the transaction"
    }

    Output:
    {
        "status": "success",
        "transaction": "Details of the successful transaction"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "mint": {"type": str, "required": True},
                "bonding_curve": {"type": str, "required": True},
                "associated_bonding_curve": {"type": str, "required": True},
                "amount": {"type": int, "required": True, "min": 1},
                "slippage": {"type": float, "required": False, "min": 0, "max": 100},
                "max_retries": {"type": int, "required": False, "min": 1}
            }
            validate_input(data, schema)

            mint = Pubkey(data["mint"])
            bonding_curve = Pubkey(data["bonding_curve"])
            associated_bonding_curve = Pubkey(data["associated_bonding_curve"])
            amount = data["amount"]
            slippage = data.get("slippage", 0.5)
            max_retries = data.get("max_retries", 3)

            result = await self.solana_kit.buy_token(
                mint, bonding_curve, associated_bonding_curve, amount, slippage, max_retries
            )
            return {
                "status": "success",
                "transaction": result.dict(),
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self):
        raise NotImplementedError("This tool only supports async execution.")



import json

from langchain.tools import BaseTool

from agentipy.agent import SolanaAgentKit
from agentipy.helpers import validate_input


class TiplinkCreateTool(BaseTool):
    name: str = "tiplink_create"
    description: str = """
    Creates a TipLink on Solana.

    Input: A JSON string with:
    {
        "amount": "float, the tip amount",
        "spl_mint_address": "string, optional, the SPL mint address"
    }

    Output:
    {
        "transaction_details": "dict, transaction details",
        "message": "string, if an error occurs"
    }
    """
    solana_kit: SolanaAgentKit

    async def _arun(self, input: str):
        try:
            data = json.loads(input)
            schema = {
                "amount": {"type": float, "required": True},
                "spl_mint_address": {"type": str, "required": False},
            }
            validate_input(data, schema)

            amount = data["amount"]
            spl_mint_address = data.get("spl_mint_address")

            result = await self.solana_kit.create_tiplink(amount, spl_mint_address)
            return {
                "status": "success",
                "transaction_details": result,
            }
        except Exception as e:
            return {
                "status": "error",
                "message": str(e),
            }

    def _run(self, input: str):
        raise NotImplementedError("This tool only supports async execution via _arun. Please use the async interface.")
